\begin{code}
module Html
where
import Edu_Berkeley_Sbp_Haskell_SBP
import FromTree
import Doc
import List(isSuffixOf,isPrefixOf)

-- FIXME: use pretty-printing when asked to for better display

class ToHtml a where
  toHtml :: a -> String
instance ToHtml a => ToHtml [a] where
  toHtml x = concatMap toHtml x

style =
  "\n<style>\n"++
  " h1, h2, h3, h4 { font-family: 'Trebuchet MS', arial, verdana, sans-serif; width: 100% }\n"++
  " h1 { font-size: 20pt; border-top: black 1px solid; }\n"++
  " h2 { font-size: 16pt; border-top: silver 1px solid; }\n"++
  " h3 { font-size: 12pt; }\n"++
  " TH, TD, P, LI, DIV, SPAN {\n"++
  "     font-family: verdana, arial, sans-serif;\n"++
  "     font-size: 12px;  \n"++
  "     text-decoration:none; \n"++
  " }\n"++
  " LI { margin-top: 5px; }\n"++
  " body { color: #333333; }\n"++
  " blockquote { font-style: italic; width: 100% }\n"++
  " div.warn { border: 1px solid red; background-color: #fbb; color: white; }\n"++
  " td.warn { color: black; }\n"++
  " div.announce { border: 1px solid green; background-color: #bfb; color: white; }\n"++
  " td.announce { color: black; }\n"++
  " div.footer {\n"++
  "   color: gray;\n"++
  "   border-top: 1px solid silver;\n"++
  "   font-size: 10px;\n"++
  " }\n"++
  " table.blockquote { margin: 5px; border: 1px #e6ddcb solid; background-color: #fbf2e0; width: 100% }\n"++
  " a:link { text-decoration: none; color: blue; border-bottom:1px dotted; }\n"++
  " a:visited { text-decoration: none; color: purple; border-bottom:1px dotted; }\n"++
  " a:active { text-decoration: none; color: red; border-bottom:1px solid; }\n"++
  " a:hover { text-decoration: none; border-bottom:1px solid; }\n"++
  " table.footer { border-top: silver solid 1px; }\n"++
  " span.signature { color: #bbb;  }\n"++
  " .signature a:link { color: #bbb;  }\n"++
  " .signature a:visited { color: #bbb;  }\n"++
  " .signature a:hover { color: blue; border-bottom: 1px solid blue;  }\n"++
  " span.highlight { background: yellow; color: black; padding: 3px }\n"++
  " div.pre {\n"++
  "     text-align: left;\n"++
  "     font-family: monospace;\n"++
  "     border-style: solid;\n"++
  "     border-width: 2px 2px 2px 2px;\n"++
  "     border-color: #6666aa;\n"++
  "     color: #FFFFFF;\n"++
  "     background-color: #000000;\n"++
  "     margin-right: 25px;\n"++
  "     margin-left: 25px;\n"++
  "     padding: 10px;\n"++
  "  }\n"++
  "</style>\n"
  

instance ToHtml Doc where
 toHtml (Doc h secs) =
     "<!-- This document was AUTOMATICALLY GENERATED from wix source -->\n"++
     "<!--    it is probably not a wise idea to edit it directly     -->\n\n"++
     "<html>\n"++
     "<head>\n"++
     style++
     --FIXME: title tag
     "</head>\n"++
     "<body>\n"++   -- tell jsmath we will escape stuff manually

     -- FIXME: only put this in if math appears on the page
     "<script> jsMath = { showFontWarnings: false } </script>\n"++
     "<script src='/jsmath/easy/load.js'></script>\n"++
     "<span id='tex2math_off'></span>\n"++
     "<NOSCRIPT> <DIV STYLE='color:#CC0000; text-align:center'> <B>Warning: <A HREF='http://www.math.union.edu/locate/jsMath'>jsMath</A> requires JavaScript to process the mathematics on this page.<BR> If your browser supports JavaScript, be sure it is enabled.</B> </DIV> <HR> </NOSCRIPT>\n"++

     "<center><table><tr><td width=600>\n"++
     (toHtml secs) ++
     "<br><br>\n"++
     "<table width=100% class=footer><tr><td align=left>"++
     "<img src='"++printIconBase64++"'></td>"++
     "<td align=right><span class='signature'>rendered from "++
     "<a href=http://www.megacz.com/software/wix>"++
     "W<span style='vertical-align:-20%'>I</span>X</a></span></div></td></tr></table>\n"++
     "</td></tr></table></center>\n"++
     "</body></html>"

quoteIconBase64 = 
    "data:image/png;base64,"
    ++"iVBORw0KGgoAAAANSUhEUgAAABAAAAAVCAYAAABPPm7SAAAABmJLR0QA/wD/AP+gvaeTAAAA"
    ++"CXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH1wQPAx0rP5obpAAAAWJJREFUOMvdlKFvg0AU"
    ++"xr8tVZ1cbU+04lUuwc2WypMX9AxiomoJ/RNYqmaaYKpb5kBys1UkWBCdKLqVs8zwFnrQmYol"
    ++"+xLC8X55H/e+wAENhWGoPM/zT6eThQ4lSWJf5EEQuAAqAJXv+95vXCkVtpwZuq4bmM1pmlrM"
    ++"LctKWzuwbTsBUNX3lph3Njfd6/WZ9vv9iHkYhsrkPa21zQ9aa9v3fRsAFovFK9cMPgIApdT7"
    ++"eDz+RB1Y1XEBwEVe54ZbXKm/N7haNwCQ5zmtVqtnE0op49lspgFgPp+/dfF/EGIPAAaDgZBS"
    ++"xgBwPB7vd7vdIwD0+/2v5ry8juNYmgY/P1GWZQ9sQER3XOcwD4fDkA2IqGiNwN8+ERVNY/Pt"
    ++"RFQIIcozg+Vy+VKW5dDcMmuz2ThFURAATKfTj7MQ1+v1Ezc7jrMVQpTmocOjOY6znUwmRSvR"
    ++"KIpknud0KfEoimSWZQ/N2jfhtb1AvGklDQAAAABJRU5ErkJggg=="

warnIconBase64 = 
    "data:image/png;base64,"
    ++"iVBORw0KGgoAAAANSUhEUgAAABQAAAAREAYAAACN1FD9AAAABmJLR0QA/wD/AP+gvaeTAAAA"
    ++"CXBIWXMAAABIAAAASABGyWs+AAAACXZwQWcAAAAUAAAAEQDeTN6UAAAD/ElEQVRIx8WUbUyV"
    ++"ZRjHf+chxDi8yZcDa0mKh/wSuNE5eNYHZBiRwoespayEkdgx22hmjB1glTlenDYlMccAX1cb"
    ++"CDFlg+aQ1JGEazGWpixdbbweDucctPF2PNxXX3pw00gL09+X59m1+77+v+d+nueCBRJ+ECAs"
    ++"78dnAPp2nLEDtMdpPwMYZhfaf8GU1gLsixwdBRDRr7lRAHmxT0zs+UKAuKmhXgDfGleAYZVh"
    ++"lXynC/Z1AYyeX+IFiLjw+MwWAxDQuBqgNVsX+iNta/nWcnV0vD+9Ir1CndDre68AfDH42PzW"
    ++"2QDW79UFxkxh6WHp8o5a5lznXCfJ/nPX867nSYArO7AssEyOjOwE8GfGZwC84Pu3edrDLgzK"
    ++"AFj06q7DAPt/1evBK4szizMlpXy4tqO2g/MHa9sm2ibwP23M78zvFE0rBAg4U24DqFpieAqA"
    ++"6kd+cjuiAAqu6Sfn/n55/fJ6CZOc6YHpATmkNWgNWoOIZtSMmlFE1d9ae2utvDt21ZRiSpHV"
    ++"+r43LwFkBT4ysehigKgDvxcC3P5QD5qxNtU11anj8hfhg+GD4YMixipjlbFK5pjyHok8Enn3"
    ++"m7yyG2DAEdIFYLy9YMHDLoBjK/WA8fhkb7JXtcqMnJbT4tJFYlpiWmJaRKJio2KjYu8KSp9K"
    ++"UAly2ZtpSbWkql/0PrsuAJR3/2cx65cA1qujRgBV6HpWy9fy5dydOz2JPYlqQO4hwZ/gT/CL"
    ++"mB1mh9kh93Enq2tP1x7V4wo0pBnS5KehVwBmPjA7AcxtDy2mBQIYPmk/B/CDb26MxGw5teWU"
    ++"+krmIdWaak21iiR2JHYkdsi83G7KtmRb1HG9b+PHAK2lJAEQ9EDBt88C5BybGyOLQgtCC8Su"
    ++"gkesI1axzRd8Mvdk7slckeoN1RuqN8wvODsyVDNUIxaXP0SFKHlfz8l4GSDTfd+B6Tdh5QCh"
    ++"hUUvAlR8o9eDI4qSipLkJcOEqdvUzaX5HmzSNGmaNMFN+037TTvgxInzb96QKTovOo/LRmuJ"
    ++"r8QnFr3+2esAB/IX9wAEjd+3cfc+gD2euTFSv2zzss0SIdnTG6c3SqU8AFulrdJWKRJSElIS"
    ++"UiLia/Y1+5r/YcNHM9tntku1e2KFZ4VHAvXcwnyAksY5MfNvAOYdQ+8BzHytL5warDtRd2I2"
    ++"QjWNh46HqjR11LvGu0aVqUOei56LYlD7PRWeCmlWpe4p95R87h66Yb9hV+7+o71BvUHKoWyj"
    ++"jlGHSlbPOTc5N6kkFTqybWSbeksxfHb4rGqd9Q3nDOdI5lRHTXtNu6rWc/vfAJjoXHoNYOl6"
    ++"Q70CaFEpYwAZhof+m/5nvv0UoDFLW/UagCXuSQvdSzwAiTv/BGXg1AxNKyCeAAAAAElFTkSu"
    ++"QmCC"

printIconBase64 = 
    "data:image/png;base64,"
    ++"iVBORw0KGgoAAAANSUhEUgAAAA8AAAAOCAMAAADHVLbdAAAALHRFWHRDcmVhdGlvbiBUaW1l"
    ++"AEZyaSAxOSBTZXAgMjAwMyAxODozOTozMiAtMDAwME2jAt8AAAAHdElNRQfTCRMRKABXeznM"
    ++"AAAACXBIWXMAAAsSAAALEgHS3X78AAAABGdBTUEAALGPC/xhBQAAACRQTFRF////AAAA7+/v"
    ++"3t7ezs7OtbW1ra2tlJSUnJycEBAQKSkphISEbGtEogAAAAF0Uk5TAEDm2GYAAABLSURBVHja"
    ++"nY1BEsAgCAMDKm31//81qB3l6h4Y4mYQQNvAmXNv7W/gytcTRj6pppQHNWoW6JWS13Ix86yr"
    ++"O7MEPkDWuWLPK/7hoYEOxksDsk8eppEAAAAASUVORK5CYII="


emailIconBase64 =
    "data:image/png;base64,"
    ++"iVBORw0KGgoAAAANSUhEUgAAABUAAAAOCAMAAAD32Kf8AAAALHRFWHRDcmVhdGlvbiBUaW1l"
    ++"AEZyaSAxOSBTZXAgMjAwMyAxODo0MjowOSAtMDAwMBDwv7IAAAAHdElNRQfTCRMRKhqYL6I0"
    ++"AAAACXBIWXMAAAsSAAALEgHS3X78AAAABGdBTUEAALGPC/xhBQAAADBQTFRF////AAAAhISE"
    ++"9/fv9/f3////CAgI7+/v1tbOxsa9tbWtpaWclJSMlJSUe3t7a2trxDv8WgAAAAF0Uk5TAEDm"
    ++"2GYAAABiSURBVHjabc9JEoAgDETRpNOKs/e/rSEYF8jfhHpQFAi6JHpHBg5VOdIj+KeXkgO9"
    ++"tUj/Bldo3WdRnktt3fbjgoWKK5EKsunkyryCn86Oxsj/UZqKEkFmNF+mvieFdSK16wFr7QK5"
    ++"tASqkwAAAABJRU5ErkJggg=="

pdfIconBase64 = 
    "data:image/png;base64,"
    ++"iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAAAK3RFWHRDcmVhdGlvbiBUaW1l"
    ++"AFRodSA2IE5vdiAyMDAzIDE1OjMwOjAwIC0wMDAwSwt8PwAAAAd0SU1FB9MLBg8fD1x8/t4A"
    ++"AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAAAQlBMVEX///9zc3Nra2uE"
    ++"hITGxsaUlJScnJx7e3uMjIz////39/fv7+/n5+fe3t69vb1jY2OlpaXW1ta1tbVaWlqtra3O"
    ++"zs5w48BYAAAAAXRSTlMAQObYZgAAAIdJREFUeNpNz1ESBBEMBNCRwSCJMOT+V10WW9vKz6uO"
    ++"iusa0R137ShxjImwHsnSMCbEX6dPiIje+SUKC6hav8CbAQlZD/RmJ1Tdz6oTwETMfAByK0hT"
    ++"1kgH7E1phFfjdvS2JlJpN8RAyEHMhgJgaexJTBPyA1JiGoD0BXgg2H8wcFs3/jDvPB+sOwir"
    ++"+o6iKQAAAABJRU5ErkJggg=="

instance ToHtml Section where
 toHtml (Section level header paragraphs) =
     "\n<h"++(show (level+1))++">\n"++
     (toHtml header)++
     "\n</h"++(show (level+1))++">\n"++
     (toHtml paragraphs)
stag t body = "\n<"++t++">\n"++body++"\n</"++t++">\n"
tag  t body = "<"++t++">"++body++"</"++t++">"

instance ToHtml Paragraph where
 toHtml (Blockquote t) = "\n<table class=blockquote border=0 cellpadding=5px>\n"
                         ++"<tr><td valign=top><image src='"++quoteIconBase64++"'></td>\n"
                         ++"<td class=warn>\n"
                         ++(toHtml t)
                         ++"</td></tr></table>\n"
 toHtml HR             = stag "hr" []
 toHtml (OL t)         = stag "ol" $ concatMap (\x -> stag "li" $ concatMap toHtml x) t
 toHtml (UL t)         = stag "ul" $ concatMap (\x -> stag "li" $ concatMap toHtml x) t
 toHtml (P t)          = stag "p"  $ toHtml t


link ref body = "<a href='"++ref++"'>"++icon++body++"</a>"
 where
  icon = if      ".pdf" `isSuffixOf` ref then "<img "++img++" src='"++pdfIconBase64++"'>&nbsp;"
         else if "mailto:" `isPrefixOf` ref then "<img "++img++" src='"++emailIconBase64++"'>&nbsp;"
         else ""
  img = "style='vertical-align: text-bottom;' border=0 "
-- margin-bottom: -2px; padding-bottom: 2px;  border-bottom: 1px blue solid;

instance ToHtml Text where
 toHtml WS                        = " "
 toHtml (Chars s)                 = toHtml s
  
 -- directional quotes: see http://www.dwheeler.com/essays/quotes-in-html.html
 toHtml (Quotes x)                = "&#8220;"++(toHtml x)++"&#8221;"
 toHtml (Verbatim x)              = pre x
 toHtml (Link t ref)              = link (show ref) (toHtml t)
 toHtml (Command "url" y)         = "<tt>"++(link (toHtml y) (toHtml y))++"</tt>"
 toHtml (Command "WiX" y)         = "W<span style='vertical-align:-20%'>I</span>X"
 toHtml (Command "TeX" y)         = "T<span style='vertical-align:-20%'>E</span>X"
--   u'1/2' : u'\u00BD',
--    u'1/4' : u'\u00BC',
--    u'3/4' : u'\u00BE',
--    u'1/3' : u'\u2153',
--    u'2/3' : u'\u2154',
--    u'1/5' : u'\u2155',
--    u'2/5' : u'\u2156',
--    u'3/5' : u'\u2157',
--    u'4/5' : u'\u2158',
--    u'1/6' : u'\u2159',
--    u'5/6' : u'\u215A',
--    u'1/8' : u'\u215B',
--    u'3/8' : u'\u215C',
--    u'5/8' : u'\u215D',
--    u'7/8' : u'\u215E',

 toHtml (Styled Underline x)      = tag "u" $ toHtml x
 toHtml (Styled TT x)             = tag "tt" $ toHtml x
 toHtml (Styled Italic x)         = tag "i" $ toHtml x
 toHtml (Styled Strikethrough x)  = tag "strike" $ toHtml x
 toHtml (Styled Superscript x)    = tag "sup" $ toHtml x
 toHtml (Styled Subscript x)      = tag "sub" $ toHtml x
 toHtml (Styled Bold x)           = tag "b" $ toHtml x
 toHtml (Styled Highlight x)      = "<span class=highlight>"++(toHtml x)++"</span>"

 toHtml (Keyword x)               = tag "tt" $ toHtml x
 toHtml (SubPar x)                = stag "p" $ concatMap toHtml x

 toHtml (Command "red" y)         = "<font color=red>"++(toHtml y)++"</font>"
 toHtml (Command "orange" y)      = "<font color=orange>"++(toHtml y)++"</font>"
 toHtml (Command "green" y)       = "<font color=green>"++(toHtml y)++"</font>"
 toHtml (Command "sc" y)          = "<sc>"++(toHtml y)++"</sc>"
 toHtml (Command "image" y)       = "<img src='"++(toHtml y)++"'/>"
 toHtml (Command "image3" y)      = "<img width=200px src='"++(toHtml y)++"'/>"
 toHtml (Command "image4" y)      = "<center><img width=550px src='"++(toHtml y)++"'/></center>"
 toHtml (Command "warn" y)        = "\n<div class=warn>\n<table border=0 cellpadding=5px>\n"
                                    ++"<tr><td valign=top><image src='"++warnIconBase64++"'></td>\n"
                                    ++"<td class=warn>\n"
                                    ++(toHtml y)
                                    ++"</td></tr></table></div>\n"
 toHtml (Command "announce" y)    = "\n<div class=announce>\n<table border=0 cellpadding=5px>\n"
                                    ++"<tr><td valign=top></td>\n"
                                    ++"<td class=warn>\n"
                                    ++(toHtml y)
                                    ++"</td></tr></table></div>\n"
 toHtml (Command "br" _)          = "\n<br/>\n"
 toHtml (Command "cent" _)        = "&#189;"
 toHtml (Command "euro" _)        = "&#8364;"

 -- gross hack
 toHtml (Command "ordinal" x) | (last (toHtml x) == '1') = (toHtml x)++"<sup>"++"st"++"</sup>"
                              | (last (toHtml x) == '2') = (toHtml x)++"<sup>"++"nd"++"</sup>"
                              | (last (toHtml x) == '3') = (toHtml x)++"<sup>"++"rd"++"</sup>"
                              | otherwise                = (toHtml x)++"<sup>"++"th"++"</sup>"
       
 -- FIXME: use "unicode vulgar fractions" here
 toHtml (Command "fraction" [n,d]) = "<sup>"++(toHtml n)++"</sup>"++"/"++"<sub>"++(toHtml d)++"</sub>"
 toHtml (Command "rfc" x)         = "<tt><a href=http://tools.ietf.org/html/rfc"++(toHtml x)++">RFC"++(toHtml x)++"</a></tt>"

 -- FIXME: add div as well (for display-mode math)
 toHtml (Math m)                  = "<span class=math>" ++ (toHtml m) ++ "</span>"
 toHtml (Footnote x)              = error $ "footnotes not supported"

 toHtml (GlyphText Euro)          = "&#8364;"
 toHtml (GlyphText CircleR)       = "&#162;"
 toHtml (GlyphText CircleC)       = "&#174;"
 toHtml (GlyphText TradeMark)     = "&#8482;"
 toHtml (GlyphText ServiceMark)   = "&#8482;"
 toHtml (GlyphText Emdash)        = "&mdash;"
 toHtml (GlyphText Ellipsis)      = "&#0133;"
 toHtml (GlyphText Cent)          = "&#189;"
 toHtml (GlyphText Daggar)        = "&#8224;"
 toHtml (GlyphText DoubleDaggar)  = "&#8225;"
 toHtml (GlyphText Clover)        = "&#8984;"
 toHtml (GlyphText Flat)          = "&#8918;"
 toHtml (GlyphText Natural)       = "&#8919;"
 toHtml (GlyphText Sharp)         = "&#8920;"
 toHtml (GlyphText CheckMark)     = "&#10003;"
 toHtml (GlyphText XMark)         = "&#10007;"
 toHtml (GlyphText LeftArrow)            = "&#;"  -- FIXME
 toHtml (GlyphText DoubleLeftArrow)      = "&#;"  -- FIXME
 toHtml (GlyphText DoubleRightArrow)     = "&#;"  -- FIXME
 toHtml (GlyphText DoubleLeftRightArrow) = "&#;"  -- FIXME
 toHtml (GlyphText LeftRightArrow)       = "&#;"  -- FIXME
 toHtml (GlyphText Degree)               = "&#;"  -- FIXME

 toHtml (Command ('k':'e':'y':'s':'t':'r':'o':'k':'e':':':k) _) =
     "&#x"++(case k of
               "command" -> "2318"
               "shift" -> "21E7"
               "option" -> "2325"
               "control" -> "2303"
               "capslock" -> "21EA"
               "apple" -> "F8FF"
               )++";"
 toHtml (Command x y)             = error $ "unsupported command "++(show x)

instance ToHtml String where
  toHtml s = concatMap htmlEscapeChar s

htmlEscapeChar '<'  = "&lt;"
htmlEscapeChar '>'  = "&gt;"
htmlEscapeChar '&'  = "&amp;"
htmlEscapeChar '\'' = "&apos;"
htmlEscapeChar '\"' = "&quot;"
htmlEscapeChar c    = [c]

pre x = "\n<div class=pre>"++ (pre' x) ++ "\n</div>\n"
 where
  pre' (' ':b)         = "&nbsp;"++(pre' b)
  pre' ('\n':b)        = "<br/>\n"++(pre' b)
  pre' (a:b)           = (htmlEscapeChar a)++(pre' b)
  pre' []              = []
\end{code}
